/*
		Initialisation de la PLL pour BT848
		===================================

Compile avec DJGPP...

Si vous voulez des infos sur la programmation du BT848, sachez que la
datasheet est facile a trouver sur le site de son constructeur...
*/



#include <stdio.h>
#include <dpmi.h>
#include <sys/farptr.h>
#include <allegro.h>

#define BROOKTREE 0x109E
#define BT848     0x0350
#define BT849     0x0351
#define BT878     0x036E
#define BT879     0x036F

#define SUPPORTEDCHIPS 4

int cree_map(unsigned long *, unsigned long, int);
int cree_selecteur(int *, unsigned long, int);
void suppr_map(unsigned long *);
void suppr_selecteur(int *);

int supportedBT [ SUPPORTEDCHIPS ] = { BT848 , BT849 , BT878 , BT879 } ;
char *Chip_Bt[] = { "BT848", "BT849", "BT878", "BT879" };

unsigned long Adr_Bt(int, int);
unsigned long AdresseBT;
unsigned long base = 0;
int segment = 0;

void main()
{
 int i, map;
 for (i = 0; i < SUPPORTEDCHIPS; i++)
   {
     AdresseBT = Adr_Bt(BROOKTREE, supportedBT[i]);
     if (AdresseBT != 0L)
	break;
   }
 if (AdresseBT != 0L)
   {
     printf("\nChipset: %s\n", Chip_Bt[i]);
     printf("Adresse: 0x%lX\n", AdresseBT);
   }
 else
   {
     printf("BTXXX non trouv...\n");
     return;
   }
 if(i >= 2)
   {
     printf("BT87X non support...\n");
     return;
   }
 map = cree_map(&base, AdresseBT, 0x20000);
 if(map == -1)
   {
     printf("Erreur cration mapping\n");
     return;
   }
 map = cree_selecteur(&segment, base, 0x20000);
 if(map == -1)
   {
     printf("Erreur cration slecteur\n");
     suppr_map(&base);
     return;
   }
 _farsetsel(segment);
 _farnspokeb(0x07C, 0xFF);              //reset de la carte
 _farnspokeb(0x0F0, 0xF9);              //F9h dans PLL_F_LO
 _farnspokeb(0x0F4, 0xDC);              //DCh dans PLL_F_HI
 _farnspokeb(0x0F8, 0x8E);              //8Eh dans PLL_XCI
 _farnspokeb(0x84, 8);                  //bit TGCKI  1
 printf("PLL initialise :-)\n");
 suppr_map(&base);
 suppr_selecteur(&segment);
}

int cree_map(unsigned long *linear, unsigned long adr_phys, int taille)
{
 __dpmi_meminfo meminfo;
 meminfo.address = adr_phys;
 meminfo.size = taille;
 if (__dpmi_physical_address_mapping(&meminfo) != 0)
    return -1;
 *linear = meminfo.address;
 __dpmi_lock_linear_region(&meminfo);
 return 0;
}

int cree_selecteur(int *segment, unsigned long linear, int taille)
{
 *segment = __dpmi_allocate_ldt_descriptors(1);
 if (*segment < 0)
   {
     *segment = 0;
     return -1;
   }
 __dpmi_set_segment_base_address(*segment, linear);
 __dpmi_set_segment_limit(*segment, MAX(taille-1, 0xFFFF));
 return 0;
}

void suppr_map(unsigned long *linear)
{
 __dpmi_meminfo meminfo;

 if (*linear)
   {
     if (*linear >= 0x100000)
       {
	 meminfo.address = *linear;
	 __dpmi_free_physical_address_mapping(&meminfo);
       }
     *linear = 0;
   }
}

void suppr_selecteur(int *segment)
{
 if (*segment)
   {
     __dpmi_free_ldt_descriptor(*segment);
     *segment = 0;
   }
}

